#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

# Database construction
dimension = 2

inputSample = ot.Sample(0, dimension)
inputSample.add(ot.Point(dimension, 1.0))
inputSample.add(ot.Point(dimension, 2.0))
inputSample.setDescription(['x0', 'x1'])
outputSample = ot.Sample(0, 1)
outputSample.add(ot.Point(1, 4.0))
outputSample.add(ot.Point(1, 5.0))
outputSample.setDescription(['y0'])
database = ot.DatabaseFunction(inputSample, outputSample)

print('database=', database)

# Does it work?
x = ot.Point(dimension, 1.8)
print('x=', x)
print('database(x)=', database(x))



# -*- coding: utf-8 -*-
"""
An optimization problem with mixed discrete input variables "4d soo".

min f(x1,x2,disc_b,logic)
where
x1 in [-5,5], x2 in [-3,3], disc_b in {True,False}, logic in {True,False}

4D function with a mixed decision space (two numeric, one discrete,
and one logical parameter)

In the original problem, we have disc_b in {"a","b"}.

Reference
Ported from R to Python.
Package ‘smoof’, August 14, 2017
Single and Multi-Objective Optimization Test Functions
Jakob Bossek
"""

import openturns as ot
import openturns.testing as ott


def fourdsoo(x):
    x1, x2, disc_b, logic = x
    # minimum : f([-5,-3,1,1]) = -18
    if round(disc_b) == 0.:
        result = x1 ** 2 + x2 ** 2 + 10 * round(logic)
    else:
        result = x1 + x2 - 10 * round(logic)
    return [result]


# Create the objective function
objectiveFun = ot.PythonFunction(4, 1, fourdsoo)

x = [-5, -3, 1, 1]

print("Evaluate f at x=", x)
print("f(x)=", objectiveFun(x))

# Define bounds
bounds = ot.Interval([-5., -3., 0., 0.], [5., 3., 1, 1])

# Define variable types
varTypes = [ot.OptimizationProblemImplementation.CONTINUOUS, ot.OptimizationProblemImplementation.CONTINUOUS,
            ot.OptimizationProblemImplementation.BINARY, ot.OptimizationProblemImplementation.INTEGER]

# Define problem
problem = ot.OptimizationProblem(objectiveFun)
problem.setVariablesType(varTypes)
problem.setBounds(bounds)
problem.setMinimization(True)

x0 = [0, 0, 0, 0]
algo = ot.Bonmin(problem, "B-BB")
algo.setStartingPoint(x0)
algo.setMaximumEvaluationNumber(10000)
algo.setMaximumIterationNumber(1000)
ot.ResourceMap.AddAsScalar('Bonmin-bonmin.time_limit', 60)
algo.run()

result = algo.getResult()
x_star = result.getOptimalPoint()
print("x*=", x_star)
y_star = result.getOptimalValue()
neval = result.getEvaluationNumber()
print("f(x*)=", y_star, "neval=", neval)


# ASSERTION
ott.assert_almost_equal(x_star, [-5, -3, 1, 1], 5e-4)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # pStudent
    nuMin = 0.2
    nuMax = 5.0
    n1 = 5
    xMin = 0.1
    xMax = 0.9
    nX = 10
    grid = [0.0]*nX
    for i1 in range(n1):
        nu = nuMin + (nuMax - nuMin) * i1 / (n1 - 1)
        for iX in range(nX):
            x = xMin + (xMax - xMin) * iX / (nX - 1)
            grid[iX] = x
            print("pStudent(", nu, ",  %.12g" % x, ")=%.6g" % DistFunc.pStudent(
                nu, x), ", complementary=%.6g" % DistFunc.pStudent(nu, x, True))
        print("pStudent(", grid, ")=", DistFunc.pStudent(nu, grid))
    # qStudent
    nuMin = 0.2
    nuMax = 5.0
    n1 = 5
    qMin = 0.1
    qMax = 0.9
    nQ = 10
    grid = [0.0]*nQ
    for i1 in range(n1):
        nu = nuMin + (nuMax - nuMin) * i1 / (n1 - 1)
        for iQ in range(nQ):
            q = qMin + (qMax - qMin) * iQ / (nQ - 1)
            grid[iQ] = q
            print("qStudent(", nu, ",  %.12g" % q, ")=%.6g" % DistFunc.qStudent(
                nu, q), ", complementary=%.6g" % DistFunc.qStudent(nu, q, True))
    print("qStudent(", grid, ")=", DistFunc.qStudent(nu, grid))
    # rStudent
    nuMin = 0.2
    nuMax = 5.0
    n1 = 5
    nR = 10
    for i1 in range(n1):
        nu = nuMin + (nuMax - nuMin) * i1 / (n1 - 1)
        for iR in range(nR):
            print("rStudent(", nu, ")=%.6g" % DistFunc.rStudent(nu))

except:
    import sys
    print("t_DistFunc_student.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
from openturns import *
from math import *

TESTPREAMBLE()
RandomGenerator.SetSeed(0)

try:
    # 1D tests
    distribution = Dirac(0.7)
    print("Distribution ", distribution)

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", oneRealization)

    # Test for sampling
    size = 10000
    oneSample = distribution.getSample(size)
    print("oneSample first=", oneSample[0], " last=", oneSample[size - 1])
    print("mean=", oneSample.computeMean())
    sampleCovariance = oneSample.computeCovariance()[0, 0]
    if (fabs(sampleCovariance) < 1.0e-16):
        sampleCovariance = 0.0
    print("covariance=", sampleCovariance)

    # Define a point
    point = Point(distribution.getDimension(), 0.0)
    print("Point= ", point)

    # Show PDF and CDF of point
    PDF = distribution.computePDF(point)
    print("pdf     =", PDF)
    print("pdf (FD)=", (distribution.computeCDF(point + Point(1, 0)) -
                        distribution.computeCDF(point + Point(1, -1))))
    CDF = distribution.computeCDF(point)
    print("cdf=", CDF)

    # Define a point
    point = Point(distribution.getSupport(distribution.getRange())[0])
    print("Point= ", point)

    # Show PDF and CDF of point
    PDF = distribution.computePDF(point)
    print("pdf     =", PDF)
    print("pdf (FD)=", (distribution.computeCDF(point + Point(1, 0)) -
                        distribution.computeCDF(point + Point(1, -1))))
    CDF = distribution.computeCDF(point)
    print("cdf=", CDF)
    CF = distribution.computeCharacteristicFunction(0.5)
    print("characteristic function= (%.12g%+.12gj)" % (CF.real, CF.imag))
    GF = distribution.computeGeneratingFunction(0.5 + 0.3j)
    print("generating function= (%.12g%+.12gj)" % (GF.real, GF.imag))
    quantile = distribution.computeQuantile(0.95)
    print("quantile=", quantile)
    print("cdf(quantile)=", distribution.computeCDF(quantile))
    print("entropy=%.6f" % distribution.computeEntropy())
    mean = distribution.getMean()
    print("mean=", mean)
    standardDeviation = distribution.getStandardDeviation()
    print("standard deviation=", standardDeviation)
    skewness = distribution.getSkewness()
    print("skewness=", skewness)
    kurtosis = distribution.getKurtosis()
    print("kurtosis=", kurtosis)
    covariance = distribution.getCovariance()
    print("covariance=", covariance)
    parameters = distribution.getParametersCollection()
    print("parameters=", parameters)
    for i in range(6):
        print("standard moment n=", i, " value=",
              distribution.getStandardMoment(i))
    print("Standard representative=", distribution.getStandardRepresentative())

    # N-D tests

    dim = 4
    distribution = Dirac(Point(dim, 2.3))
    print("Distribution ", distribution)

    # Is this distribution elliptical ?
    print("Elliptical = ", distribution.isElliptical())

    # Is this distribution continuous ?
    print("Continuous = ", distribution.isContinuous())

    # Test for realization of distribution
    oneRealization = distribution.getRealization()
    print("oneRealization=", oneRealization)

    # Test for sampling
    size = 10000
    oneSample = distribution.getSample(size)
    print("oneSample first=", oneSample[0], " last=", oneSample[size - 1])
    print("mean=", oneSample.computeMean())
    sampleCovariance = oneSample.computeCovariance()
    for i in range(dim):
        for j in range(i + 1):
            if (fabs(sampleCovariance[i, j]) < 1.0e-16):
                sampleCovariance[i, j] = 0.0
    print("covariance=", sampleCovariance)

    # Define a point
    point = Point(dim, 0.0)
    print("Point= ", point)

    # Show PDF and CDF of point
    PDF = distribution.computePDF(point)
    print("pdf     =", PDF)
    print("pdf (FD)=", (distribution.computeCDF(point + Point(dim, 0)) -
                        distribution.computeCDF(point + Point(dim, -1))))
    CDF = distribution.computeCDF(point)
    print("cdf=", CDF)

    # Define a point
    point = Point(distribution.getSupport(distribution.getRange())[0])
    print("Point= ", point)

    # Show PDF and CDF of point
    PDF = distribution.computePDF(point)
    print("pdf     =", PDF)
    print("pdf (FD)=", (distribution.computeCDF(point + Point(dim, 0)) -
                        distribution.computeCDF(point + Point(dim, -1))))
    CDF = distribution.computeCDF(point)
    print("cdf=", CDF)

    quantile = distribution.computeQuantile(0.95)
    print("quantile=", quantile)
    print("cdf(quantile)=", distribution.computeCDF(quantile))
    mean = distribution.getMean()
    print("mean=", mean)
    standardDeviation = distribution.getStandardDeviation()
    print("standard deviation=", standardDeviation)
    skewness = distribution.getSkewness()
    print("skewness=", skewness)
    kurtosis = distribution.getKurtosis()
    print("kurtosis=", kurtosis)
    covariance = distribution.getCovariance()
    print("covariance=", covariance)
    parameters = distribution.getParametersCollection()
    print("parameters=", parameters)
    for i in range(6):
        print("standard moment n=", i, " value=",
              distribution.getStandardMoment(i))
    print("Standard representative=", distribution.getStandardRepresentative())

except:
    import sys
    print("t_Dirac.py", sys.exc_info()[0], sys.exc_info()[1])



#! /usr/bin/env python

from __future__ import print_function
import openturns as ot

# Tests
ot.ResourceMap.SetAsUnsignedInteger(
    'SimulationAlgorithm-DefaultBlockSize', 100)
ot.ResourceMap.SetAsUnsignedInteger(
    'SimulationAlgorithm-DefaultMaximumOuterSampling', 100)
ot.ResourceMap.SetAsScalar(
    'SimulationAlgorithm-DefaultMaximumCoefficientOfVariation', 0.0)
ot.ResourceMap.SetAsScalar(
    'SimulationAlgorithm-DefaultMaximumStandardDeviation', 0.0)
ot.ResourceMap.SetAsScalar(
    'RootStrategy-DefaultStepSize', 0.1)

algorithms = ['MonteCarlo',
              'LHS',
              'QuasiMonteCarlo',
              'DirectionalSampling']

inDim = 4
X = ot.RandomVector(ot.Normal(inDim))
inVars = ot.Description.BuildDefault(inDim, 'x')

low = 1.0
up = 2.0
intervals = [ot.Interval([low], [up], [True],  [False]),
             ot.Interval([low], [up], [False], [True]),
             ot.Interval([low], [up], [True],  [True]),
             ot.Interval([low], [up], [False], [False]),
             ot.Interval([low] * 2, [up] * 2, [True, True],   [True, True]),
             ot.Interval([low] * 2, [up] * 2, [True, True],   [True, False]),
             ot.Interval([low] * 2, [up] * 2, [True, True],   [False, True]),
             ot.Interval([low] * 2, [up] * 2, [True, True],   [False, False]),
             ot.Interval([low] * 2, [up] * 2, [True, False],  [True, True]),
             ot.Interval([low] * 2, [up] * 2, [True, False],  [True, False]),
             ot.Interval([low] * 2, [up] * 2, [True, False],  [False, True]),
             ot.Interval([low] * 2, [up] * 2, [True, False],  [False, False]),
             ot.Interval([low] * 2, [up] * 2, [False, True],  [True, True]),
             ot.Interval([low] * 2, [up] * 2, [False, True],  [True, False]),
             ot.Interval([low] * 2, [up] * 2, [False, True],  [False, True]),
             ot.Interval([low] * 2, [up] * 2, [False, True],  [False, False]),
             ot.Interval([low] * 2, [up] * 2, [False, False], [True, True]),
             ot.Interval([low] * 2, [up] * 2, [False, False], [True, False]),
             ot.Interval([low] * 2, [up] * 2, [False, False], [False, True]),
             ot.Interval([low] * 2, [up] * 2, [False, False], [False, False])]

for domain in intervals:
    print('#' * 50)
    print('domain=\n', domain)
    outDim = domain.getDimension()
    f = ot.SymbolicFunction(inVars, inVars[0:outDim])
    Y = ot.CompositeRandomVector(f, X)
    event = ot.ThresholdEvent(Y, domain)

    ot.RandomGenerator.SetSeed(0)
    # algo = getattr(openturns, algoName)(event)
    algo = ot.ProbabilitySimulationAlgorithm(event, ot.MonteCarloExperiment())
    algo.run()
    res = algo.getResult().getProbabilityEstimate()
    print('MC p=%.6g' % res)

    ot.RandomGenerator.SetSeed(0)
    # algo = getattr(openturns, algoName)(event)
    algo = ot.FORM(ot.Cobyla(), event, X.getMean())
    algo.run()
    res = algo.getResult().getEventProbability()
    print('FORM p=%.2f' % res)



#! /usr/bin/env python

from __future__ import print_function
from openturns import *

TESTPREAMBLE()

try:
    charlier = CharlierFactory(2.5)
    print("charlier=", charlier)
    for i in range(10):
        print("charlier(", i, ")=", charlier.build(i))
    roots = charlier.getRoots(10)
    print("charlier(10) roots=", repr(roots))
    nodes, weights = charlier.getNodesAndWeights(10)
    print("charlier(10) nodes=", nodes, "and weights=", weights)
except:
    import sys
    print("t_CharlierFactory_std.py", sys.exc_info()[0], sys.exc_info()[1])
